@@ -30,6 +30,7 @@
 #include "accounts/AccountManager.h"
 #include "TomahawkSettings.h"
 #include "utils/TomahawkUtilsGui.h"
+#include "sip/PeerInfo.h"
 
 #include "config.h"
 #include "TomahawkVersion.h"
@@ -119,7 +120,7 @@ XmppSipPlugin::XmppSipPlugin( Account* account )
 
 #ifndef ENABLE_HEADLESS
     // instantiate XmlConsole
-    if( readXmlConsoleEnabled() )
+    if ( readXmlConsoleEnabled() )
     {
         m_xmlConsole = new XmlConsole( m_client );
         m_xmlConsole->show();
@@ -138,7 +139,7 @@ XmppSipPlugin::XmppSipPlugin( Account* account )
 #endif
 
     // setup disco
-    m_client->disco()->setSoftwareVersion( "Tomahawk Player", TOMAHAWK_VERSION, CMAKE_SYSTEM );
+    m_client->disco()->setSoftwareVersion( "Tomahawk Player", TOMAHAWK_VERSION, TOMAHAWK_SYSTEM );
     m_client->disco()->addIdentity( Jreen::Disco::Identity( "client", "type", "tomahawk", "en" ) );
     m_client->disco()->addFeature( TOMAHAWK_FEATURE );
 
@@ -191,6 +192,7 @@ XmppSipPlugin::~XmppSipPlugin()
     delete m_client;
 }
 
+
 QString
 XmppSipPlugin::inviteString() const
 {
@@ -202,7 +204,7 @@ InfoSystem::InfoPluginPtr
 XmppSipPlugin::infoPlugin()
 {
     if ( m_infoPlugin.isNull() )
-        m_infoPlugin = QWeakPointer< Tomahawk::InfoSystem::XmppInfoPlugin >( new Tomahawk::InfoSystem::XmppInfoPlugin( this ) );
+        m_infoPlugin = QPointer< Tomahawk::InfoSystem::XmppInfoPlugin >( new Tomahawk::InfoSystem::XmppInfoPlugin( this ) );
 
     return InfoSystem::InfoPluginPtr( m_infoPlugin.data() );
 }
@@ -238,7 +240,7 @@ XmppSipPlugin::connectPlugin()
     QTimer::singleShot( 1000, m_client, SLOT( connectToServer() ) );
 
     if ( m_client->connection() )
-        connect( m_client->connection(), SIGNAL( error( Jreen::Connection::SocketError ) ), SLOT( onError( Jreen::Connection::SocketError ) ) );
+        connect( m_client->connection(), SIGNAL( error( Jreen::Connection::SocketError ) ), SLOT( onError( Jreen::Connection::SocketError ) ), Qt::UniqueConnection );
 
     m_state = Account::Connecting;
     emit stateChanged( m_state );
@@ -271,6 +273,8 @@ XmppSipPlugin::disconnectPlugin()
     emit stateChanged( m_state );
 
     m_client->disconnectFromServer( true );
+
+    setAllPeersOffline();
 }
 
 
@@ -322,6 +326,10 @@ XmppSipPlugin::onDisconnect( Jreen::Client::DisconnectReason reason )
     switch( reason )
     {
         case Jreen::Client::User:
+            foreach( const Jreen::JID &peer, m_peers.keys() )
+            {
+                handlePeerStatus( peer, Jreen::Presence::Unavailable );
+            }
             break;
 
         case Jreen::Client::AuthorizationError:
@@ -353,11 +361,6 @@ XmppSipPlugin::onDisconnect( Jreen::Client::DisconnectReason reason )
 
     removeMenuHelper();
 
-    Q_FOREACH( const Jreen::JID &peer, m_peers.keys() )
-    {
-        handlePeerStatus( peer, Jreen::Presence::Unavailable );
-    }
-
     if ( !m_infoPlugin.isNull() )
         Tomahawk::InfoSystem::InfoSystem::instance()->removeInfoPlugin( infoPlugin() );
 }
@@ -432,9 +435,9 @@ XmppSipPlugin::errorMessage( Jreen::Client::DisconnectReason reason )
 
 
 void
-XmppSipPlugin::sendMsg( const QString& to, const SipInfo& info )
+XmppSipPlugin::sendSipInfo( const Tomahawk::peerinfo_ptr& receiver, const SipInfo& info )
 {
-    qDebug() << Q_FUNC_INFO << to << info;
+    tDebug( LOGVERBOSE ) << Q_FUNC_INFO << receiver << info;
 
     if ( !m_client )
         return;
@@ -442,13 +445,13 @@ XmppSipPlugin::sendMsg( const QString& to, const SipInfo& info )
     TomahawkXmppMessage *sipMessage;
     if ( info.isVisible() )
     {
-        sipMessage = new TomahawkXmppMessage( info.host(), info.port(), info.uniqname(), info.key() );
+        sipMessage = new TomahawkXmppMessage( info.host(), info.port(), info.nodeId(), info.key() );
     }
     else
         sipMessage = new TomahawkXmppMessage();
 
-    qDebug() << "Send sip messsage to" << to;
-    Jreen::IQ iq( Jreen::IQ::Set, to );
+    qDebug() << "Send sip messsage to" << receiver;
+    Jreen::IQ iq( Jreen::IQ::Set, receiver->id() );
     iq.addExtension( sipMessage );
     Jreen::IQReply *reply = m_client->send( iq );
     reply->setData( SipMessageSent );
@@ -460,11 +463,16 @@ void
 XmppSipPlugin::addContact( const QString& jid, const QString& msg )
 {
     // Add contact to the Tomahawk group on the roster
-    QString realJid = jid;
-    if ( !realJid.contains( '@' ) )
-        realJid += defaultSuffix();
-
-    m_roster->subscribe( realJid, msg, realJid, QStringList() << "Tomahawk" );
+    QStringList jidParts = jid.split( '@' );
+    if ( jidParts.count() == 2 && !jidParts[0].trimmed().isEmpty() && !jidParts[1].trimmed().isEmpty() )
+    {
+        m_roster->subscribe( jid, msg, jid, QStringList() << "Tomahawk" );
+        emit inviteSentSuccess( jid );
+    }
+    else
+    {
+        emit inviteSentFailure( jid );
+    }
 }
 
 
@@ -689,13 +697,10 @@ XmppSipPlugin::onNewMessage( const Jreen::Message& message )
 
         // this is not a sip message, so we send it directly through the client
         m_client->send( Jreen::Message ( Jreen::Message::Error, Jreen::JID( to ), response) );
-
-        emit msgReceived( from, msg );
         return;
     }
 
     qDebug() << Q_FUNC_INFO << "From:" << message.from().full() << ":" << message.body();
-    emit sipInfoReceived( from, info );
 }
 
 
@@ -709,7 +714,7 @@ XmppSipPlugin::onPresenceReceived( const Jreen::RosterItem::Ptr& item, const Jre
     Jreen::JID jid = presence.from();
     QString fulljid( jid.full() );
 
-    qDebug() << Q_FUNC_INFO << "* New presence:" << fulljid << presence.subtype();
+    tDebug( LOGVERBOSE ) << Q_FUNC_INFO << "New presence:" << fulljid << presence.subtype();
 
     if ( jid == m_client->jid() )
         return;
@@ -720,11 +725,29 @@ XmppSipPlugin::onPresenceReceived( const Jreen::RosterItem::Ptr& item, const Jre
         return;
     }
 
+    // cache name
+    if( !item.isNull() && item->name() != jid.bare() && m_jidsNames.value( jid.bare() ) != item->name() )
+    {
+        tLog( LOGVERBOSE ) << Q_FUNC_INFO << "Cache name" << item->name() << "for" << jid.bare() << item << presence.subtype();
+        m_jidsNames.insert( jid.bare(), item->name() );
+
+        // find peers for the jid and update their friendlyName
+        foreach ( const Jreen::JID& peer, m_peers.keys() )
+        {
+            if ( peer.bare() == jid.bare() )
+            {
+                Tomahawk::peerinfo_ptr peerInfo = PeerInfo::get( this, peer.full() );
+                if( !peerInfo.isNull() )
+                    peerInfo->setFriendlyName( item->name() );
+            }
+        }
+    }
+
     // ignore anyone not Running tomahawk:
     Jreen::Capabilities::Ptr caps = presence.payload<Jreen::Capabilities>();
     if ( caps )
     {
-        qDebug() << Q_FUNC_INFO << fulljid << "Running tomahawk: maybe" << "caps" << caps->node() << "requesting disco...";
+        tDebug( LOGVERBOSE ) << Q_FUNC_INFO << fulljid << "Running tomahawk: maybe" << "caps" << caps->node() << "requesting disco...";
 
         // request disco features
         QString node = caps->node() + '#' + caps->ver();
@@ -857,37 +880,41 @@ XmppSipPlugin::onNewIq( const Jreen::IQ& iq )
             handlePeerStatus( jid, Jreen::Presence::Available );
         }
     }
-    else if ( context == RequestVersion)
+    else if ( context == RequestVersion )
     {
         Jreen::SoftwareVersion::Ptr softwareVersion = iq.payload<Jreen::SoftwareVersion>();
         if ( softwareVersion )
         {
             QString versionString = QString( "%1 %2 %3" ).arg( softwareVersion->name(), softwareVersion->os(), softwareVersion->version() );
             qDebug() << Q_FUNC_INFO << "Received software version for" << iq.from().full() << ":" << versionString;
-            emit softwareVersionReceived( iq.from().full(), versionString );
+            Tomahawk::peerinfo_ptr peerInfo =  PeerInfo::get( this, iq.from().full() );
+            if ( !peerInfo.isNull() )
+            {
+                peerInfo->setVersionString( versionString );
+            }
         }
     }
     else if ( context == RequestedDisco )
     {
         qDebug() << "Sent IQ(Set), what should be happening here?";
     }
-    else if( context == SipMessageSent )
+    else if ( context == SipMessageSent )
     {
         qDebug() << "Sent SipMessage... what now?!";
     }
-    /*else if(context == RequestedVCard )
+    /*else if ( context == RequestedVCard )
     {
         qDebug() << "Requested VCard... what now?!";
     }*/
     else
     {
         TomahawkXmppMessage::Ptr sipMessage = iq.payload< TomahawkXmppMessage >();
-        if(sipMessage)
+        if ( sipMessage )
         {
             iq.accept();
 
             qDebug() << Q_FUNC_INFO << "Got SipMessage ..."
-                     << "ip" << sipMessage->ip() << "port" << sipMessage->port() << "uniqname" << sipMessage->uniqname() << "key" << sipMessage->key() << "visible" << sipMessage->visible();
+                     << "ip" << sipMessage->ip() << "port" << sipMessage->port() << "nodeId" << sipMessage->uniqname() << "key" << sipMessage->key() << "visible" << sipMessage->visible();
 
             SipInfo info;
             info.setVisible( sipMessage->visible() );
@@ -895,14 +922,20 @@ XmppSipPlugin::onNewIq( const Jreen::IQ& iq )
             {
                 info.setHost( sipMessage->ip() );
                 info.setPort( sipMessage->port() );
-                info.setUniqname( sipMessage->uniqname() );
+                info.setNodeId( sipMessage->uniqname() );
                 info.setKey( sipMessage->key() );
             }
 
             Q_ASSERT( info.isValid() );
 
             qDebug() << Q_FUNC_INFO << "From:" << iq.from().full() << ":" << info;
-            emit sipInfoReceived( iq.from().full(), info );
+            Tomahawk::peerinfo_ptr peerInfo = PeerInfo::get( this, iq.from().full() );
+            if ( peerInfo.isNull() )
+            {
+                tDebug() << Q_FUNC_INFO << "no valid peerInfo for" << iq.from().full();
+                return;
+            }
+            peerInfo->setSipInfo( info );
         }
     }
 }
@@ -930,14 +963,23 @@ XmppSipPlugin::handlePeerStatus( const Jreen::JID& jid, Jreen::Presence::Type pr
 {
     QString fulljid = jid.full();
 
+    if ( fulljid.contains( "public.talk.google.com" ) )
+        return;
+
     // "going offline" event
     if ( !presenceMeansOnline( presenceType ) &&
        ( !m_peers.contains( jid ) || presenceMeansOnline( m_peers.value( jid ) ) ) )
     {
-        m_peers[ jid ] = presenceType;
         qDebug() << Q_FUNC_INFO << "* Peer goes offline:" << fulljid;
 
-        emit peerOffline( fulljid );
+        m_peers[ jid ] = presenceType;
+
+        Tomahawk::peerinfo_ptr peerInfo = PeerInfo::get( this, fulljid );
+        if ( !peerInfo.isNull() )
+        {
+            peerInfo->setStatus( PeerInfo::Offline );
+        }
+
         return;
     }
 
@@ -945,10 +987,14 @@ XmppSipPlugin::handlePeerStatus( const Jreen::JID& jid, Jreen::Presence::Type pr
     if ( presenceMeansOnline( presenceType ) &&
        ( !m_peers.contains( jid ) || !presenceMeansOnline( m_peers.value( jid ) ) ) )
     {
-        m_peers[ jid ] = presenceType;
         qDebug() << Q_FUNC_INFO << "* Peer goes online:" << fulljid;
 
-        emit peerOnline( fulljid );
+        m_peers[ jid ] = presenceType;
+
+        Tomahawk::peerinfo_ptr peerInfo = PeerInfo::get( this, fulljid, PeerInfo::AutoCreate );
+        peerInfo->setContactId( jid.bare() );
+        peerInfo->setStatus( PeerInfo::Online );
+        peerInfo->setFriendlyName( m_jidsNames.value( jid.bare() ) );
 
 #ifndef ENABLE_HEADLESS
         if ( !m_avatarManager->avatar( jid.bare() ).isNull() )
@@ -986,16 +1032,16 @@ XmppSipPlugin::onNewAvatar( const QString& jid )
     {
         if ( peer.bare() == jid )
         {
-            emit avatarReceived( peer.full(), m_avatarManager->avatar( jid ) );
+            Tomahawk::peerinfo_ptr peerInfo = PeerInfo::get( this, peer.full() );
+            if ( !peerInfo.isNull() )
+                peerInfo->setAvatar( m_avatarManager->avatar( jid ) );
         }
     }
 
     if ( jid == m_client->jid().bare() )
-        // own avatar
-        emit avatarReceived( m_avatarManager->avatar( jid ) );
-    else
-        // someone else's avatar
-        emit avatarReceived( jid,  m_avatarManager->avatar( jid ) );
+    {
+        PeerInfo::getSelf( this, PeerInfo::AutoCreate )->setAvatar( m_avatarManager->avatar( jid ) );
+    }
 #endif
 }
 
