@@ -2,6 +2,7 @@
  *
  *   Copyright 2010-2011, Christian Muehlhaeuser <muesli@tomahawk-player.org>
  *   Copyright 2010-2012, Jeff Mitchell <jeff@tomahawk-player.org>
+ *   Copyright 2013,      Teo Mrnjavac <teo@kde.org>
  *
  *   Tomahawk is free software: you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -20,7 +21,7 @@
 #include "Artist.h"
 
 #include "ArtistPlaylistInterface.h"
-#include "Collection.h"
+#include "collection/Collection.h"
 #include "database/Database.h"
 #include "database/DatabaseImpl.h"
 #include "database/DatabaseCommand_AllAlbums.h"
@@ -33,19 +34,17 @@
 
 #include <QReadWriteLock>
 
-#define ID_THREAD_DEBUG 0
-
 using namespace Tomahawk;
 
-QHash< QString, artist_ptr > Artist::s_artistsByName = QHash< QString, artist_ptr >();
-QHash< unsigned int, artist_ptr > Artist::s_artistsById = QHash< unsigned int, artist_ptr >();
+QHash< QString, artist_wptr > Artist::s_artistsByName = QHash< QString, artist_wptr >();
+QHash< unsigned int, artist_wptr > Artist::s_artistsById = QHash< unsigned int, artist_wptr >();
 
 static QMutex s_nameCacheMutex;
-static QMutex s_idCacheMutex;
 static QReadWriteLock s_idMutex;
 
 Artist::~Artist()
 {
+    tDebug( LOGVERBOSE ) << Q_FUNC_INFO << "Deleting artist:" << m_name;
     m_ownRef.clear();
 
 #ifndef ENABLE_HEADLESS
@@ -60,23 +59,22 @@ Artist::get( const QString& name, bool autoCreate )
     if ( name.isEmpty() )
         return artist_ptr();
 
-    const QString sortname = name.toLower();
-
     QMutexLocker lock( &s_nameCacheMutex );
-    if ( s_artistsByName.contains( sortname ) )
-        return s_artistsByName.value( sortname );
+    const QString key = name.toLower();
+    if ( s_artistsByName.contains( key ) )
+    {
+        artist_wptr artist = s_artistsByName.value( key );
+        if ( !artist.isNull() )
+            return artist.toStrongRef();
+    }
 
     if ( !Database::instance() || !Database::instance()->impl() )
         return artist_ptr();
 
-#if ID_THREAD_DEBUG
-        tDebug() << "Creating artist:" << name << "( sortname:" << sortname << ")";
-#endif
-
-    artist_ptr artist = artist_ptr( new Artist( name ), &QObject::deleteLater );
+    artist_ptr artist = artist_ptr( new Artist( name ), &Artist::deleteLater );
     artist->setWeakRef( artist.toWeakRef() );
     artist->loadId( autoCreate );
-    s_artistsByName.insert( sortname, artist );
+    s_artistsByName.insert( key, artist );
 
     return artist;
 }
@@ -85,25 +83,35 @@ Artist::get( const QString& name, bool autoCreate )
 artist_ptr
 Artist::get( unsigned int id, const QString& name )
 {
-    QMutexLocker lock( &s_idCacheMutex );
-
-    const QString sortname = name.toLower();
-    if ( s_artistsByName.contains( sortname ) )
-    {
-        return s_artistsByName.value( sortname );
-    }
+    s_idMutex.lockForRead();
     if ( s_artistsById.contains( id ) )
     {
-        return s_artistsById.value( id );
+        artist_wptr artist = s_artistsById.value( id );
+        s_idMutex.unlock();
+
+        if ( !artist.isNull() )
+            return artist;
+    }
+    s_idMutex.unlock();
+
+    QMutexLocker lock( &s_nameCacheMutex );
+    const QString key = name.toLower();
+    if ( s_artistsByName.contains( key ) )
+    {
+        artist_wptr artist = s_artistsByName.value( key );
+        if ( !artist.isNull() )
+            return artist;
     }
 
-    artist_ptr a = artist_ptr( new Artist( id, name ), &QObject::deleteLater );
+    artist_ptr a = artist_ptr( new Artist( id, name ), &Artist::deleteLater );
     a->setWeakRef( a.toWeakRef() );
+    s_artistsByName.insert( key, a );
 
-    s_artistsByName.insert( sortname, a );
     if ( id > 0 )
     {
+        s_idMutex.lockForWrite();
         s_artistsById.insert( id, a );
+        s_idMutex.unlock();
     }
 
     return a;
@@ -124,6 +132,7 @@ Artist::Artist( unsigned int id, const QString& name )
     , m_cover( 0 )
 #endif
 {
+    tDebug( LOGVERBOSE ) << Q_FUNC_INFO << "Creating artist:" << id << name;
     m_sortname = DatabaseImpl::sortname( name, true );
 }
 
@@ -142,10 +151,36 @@ Artist::Artist( const QString& name )
     , m_cover( 0 )
 #endif
 {
+    tDebug( LOGVERBOSE ) << Q_FUNC_INFO << "Creating artist:" << name;
     m_sortname = DatabaseImpl::sortname( name, true );
 }
 
 
+void
+Artist::deleteLater()
+{
+    QMutexLocker lock( &s_nameCacheMutex );
+
+    const QString key = m_name.toLower();
+    if ( s_artistsByName.contains( key ) )
+    {
+        s_artistsByName.remove( key );
+    }
+
+    if ( m_id > 0 )
+    {
+        s_idMutex.lockForWrite();
+        if ( s_artistsById.contains( m_id ) )
+        {
+            s_artistsById.remove( m_id );
+        }
+        s_idMutex.unlock();
+    }
+
+    QObject::deleteLater();
+}
+
+
 void
 Artist::onTracksLoaded( Tomahawk::ModelMode mode, const Tomahawk::collection_ptr& collection )
 {
@@ -165,13 +200,29 @@ Artist::albums( ModelMode mode, const Tomahawk::collection_ptr& collection ) con
 
     if ( ( mode == DatabaseMode || mode == Mixed ) && !dbLoaded )
     {
-        DatabaseCommand_AllAlbums* cmd = new DatabaseCommand_AllAlbums( collection, artist );
-        cmd->setData( QVariant( collection.isNull() ) );
+        if ( collection.isNull() )
+        {
+            DatabaseCommand_AllAlbums* cmd = new DatabaseCommand_AllAlbums( collection, artist );
+            cmd->setData( QVariant( collection.isNull() ) );
 
-        connect( cmd, SIGNAL( albums( QList<Tomahawk::album_ptr>, QVariant ) ),
-                        SLOT( onAlbumsFound( QList<Tomahawk::album_ptr>, QVariant ) ) );
+            connect( cmd, SIGNAL( albums( QList<Tomahawk::album_ptr>, QVariant ) ),
+                          SLOT( onAlbumsFound( QList<Tomahawk::album_ptr>, QVariant ) ) );
 
-        Database::instance()->enqueue( QSharedPointer<DatabaseCommand>( cmd ) );
+            Database::instance()->enqueue( QSharedPointer<DatabaseCommand>( cmd ) );
+        }
+        else
+        {
+            //collection is *surely* not null, and might be a ScriptCollection
+            Tomahawk::AlbumsRequest* cmd = collection->requestAlbums( artist );
+
+            // There is also a signal albums( QList, QVariant ).
+            // The QVariant might carry a bool that says whether the dbcmd was executed for a null collection
+            // but here we know for a fact that the collection is not null, so we'll happily ignore it
+            connect( dynamic_cast< QObject* >( cmd ), SIGNAL( albums( QList<Tomahawk::album_ptr> ) ),
+                     this, SLOT( onAlbumsFound( QList<Tomahawk::album_ptr> ) ) );
+
+            cmd->enqueue();
+        }
     }
 
     if ( ( mode == InfoSystemMode || mode == Mixed ) && !infoLoaded )
@@ -269,25 +320,19 @@ Artist::id() const
 
     if ( waiting )
     {
-#if ID_THREAD_DEBUG
-        qDebug() << Q_FUNC_INFO << "Asked for artist ID and NOT loaded yet" << m_name << m_idFuture.isFinished();
-#endif
+//        qDebug() << Q_FUNC_INFO << "Asked for artist ID and NOT loaded yet" << m_name << m_idFuture.isFinished();
         m_idFuture.waitForFinished();
-#if ID_THREAD_DEBUG
-        qDebug() << "DONE WAITING:" << m_idFuture.resultCount() << m_idFuture.isResultReadyAt(0) << m_idFuture.isCanceled() << m_idFuture.isFinished() << m_idFuture.isPaused() << m_idFuture.isRunning() << m_idFuture.isStarted();
-#endif
+//        qDebug() << "DONE WAITING:" << m_idFuture.resultCount() << m_idFuture.isResultReadyAt( 0 ) << m_idFuture.isCanceled() << m_idFuture.isFinished() << m_idFuture.isPaused() << m_idFuture.isRunning() << m_idFuture.isStarted();
         finalid = m_idFuture.result();
 
-#if ID_THREAD_DEBUG
-        qDebug() << Q_FUNC_INFO << "Got loaded artist:" << m_name << finalid;
-#endif
+//        qDebug() << Q_FUNC_INFO << "Got loaded artist:" << m_name << finalid;
 
         s_idMutex.lockForWrite();
         m_id = finalid;
         m_waitingForFuture = false;
 
         if ( m_id > 0 )
-            s_artistsById[ m_id ] = m_ownRef.toStrongRef();
+            s_artistsById.insert( m_id, m_ownRef.toStrongRef() );
 
         s_idMutex.unlock();
     }
@@ -376,9 +421,9 @@ Artist::playbackCount( const source_ptr& source )
 
 
 void
-Artist::onAlbumsFound( const QList< album_ptr >& albums, const QVariant& data )
+Artist::onAlbumsFound( const QList< album_ptr >& albums, const QVariant& collectionIsNull )
 {
-    if ( data.toBool() )
+    if ( collectionIsNull.toBool() )
     {
         m_databaseAlbums << albums;
         m_albumsLoaded.insert( DatabaseMode, true );
