@@ -24,6 +24,12 @@
 #include "Source.h"
 #include "config.h"
 
+#include "utils/Logger.h"
+#include "accounts/ResolverAccount.h"
+#include "accounts/AccountManager.h"
+#include "utils/BinaryInstallerHelper.h"
+#include "utils/Closure.h"
+
 #include <attica/downloaditem.h>
 
 #include <QCoreApplication>
@@ -35,12 +41,6 @@
 #include <QDomElement>
 #include <QDomNode>
 
-#include "utils/Logger.h"
-#include "accounts/ResolverAccount.h"
-#include "accounts/AccountManager.h"
-#include "utils/BinaryInstallerHelper.h"
-#include "utils/Closure.h"
-
 using namespace Attica;
 
 AtticaManager* AtticaManager::s_instance = 0;
@@ -391,11 +391,11 @@ AtticaManager::resolversList( BaseJob* j )
 
     fetchMissingIcons();
 
-    syncServerData();
 
     if ( ++m_resolverJobsLoaded == 2 )
     {
         qSort( m_resolvers.begin(), m_resolvers.end(), resolverSort );
+        syncServerData();
         emit resolversLoaded( m_resolvers );
     }
 }
@@ -449,6 +449,7 @@ AtticaManager::binaryResolversList( BaseJob* j )
     if ( ++m_resolverJobsLoaded == 2 )
     {
         qSort( m_resolvers.begin(), m_resolvers.end(), resolverSort );
+        syncServerData();
         emit resolversLoaded( m_resolvers );
     }
 }
@@ -502,6 +503,7 @@ AtticaManager::syncServerData()
             }
 
             // DO we need to upgrade?
+            tDebug( LOGVERBOSE ) << "Upgrade check for" << upstream.id() << "local is" << r.version << "upstream is" << upstream.version() << "and state is: " << r.state;
             if ( ( r.state == Installed || r.state == NeedsUpgrade ) &&
                  !upstream.version().isEmpty() )
             {
@@ -546,7 +548,8 @@ void AtticaManager::doInstallResolver( const Content& resolver, bool autoCreate,
 
 //    ItemJob< DownloadItem >* job = m_resolverProvider.downloadLink( resolver.id() );
     QUrl url( QString( "%1/resolvers/v1/content/download/%2/1" ).arg( hostname() ).arg( resolver.id() ) );
-    url.addQueryItem( "tomahawkversion", TomahawkUtils::appFriendlyVersion() );
+
+    TomahawkUtils::urlAddQueryItem( url, "tomahawkversion", TomahawkUtils::appFriendlyVersion() );
     QNetworkReply* r = TomahawkUtils::nam()->get( QNetworkRequest( url ) );
     NewClosure( r, SIGNAL( finished() ), this, SLOT( resolverDownloadFinished( QNetworkReply* ) ), r );
     r->setProperty( "resolverId", resolver.id() );
@@ -702,6 +705,7 @@ AtticaManager::payloadFetched()
 
     if ( installedSuccessfully )
     {
+        tDebug( LOGVERBOSE ) << "Setting installed state to resolver:" << resolverId;
         m_resolverStates[ resolverId ].state = Installed;
         TomahawkSettingsGui::instanceGui()->setAtticaResolverStates( m_resolverStates );
         emit resolverInstalled( resolverId );
