@@ -242,7 +242,8 @@ AccountDelegate::paint ( QPainter* painter, const QStyleOptionViewItem& option,
         {
             Q_ASSERT( accts.size() == 1 );
 
-            rightEdge = drawStatus( painter, QPointF( rightEdge, center - painter->fontMetrics().height()/2 ), accts.first(), true );
+            painter->setFont( installFont );
+            rightEdge = drawStatus( painter, QPointF( rightEdge - PADDING, center - painter->fontMetrics().height()/2 ), accts.first(), true );
         }
     }
     else if ( canDelete )
@@ -278,11 +279,16 @@ AccountDelegate::paint ( QPainter* painter, const QStyleOptionViewItem& option,
     // Draw the title and description
     // title
     QString title = index.data( Qt::DisplayRole ).toString();
+    QString author = index.data( AccountModel::AuthorRole ).toString();
+    QString desc = index.data( AccountModel::DescriptionRole ).toString();
+
     const int rightTitleEdge = rightEdge - PADDING;
     const int leftTitleEdge = pixmapRect.right() + PADDING;
     painter->setFont( titleFont );
     QRect textRect;
-    const bool canRate = index.data( AccountModel::CanRateRole ).toBool();
+    const bool canRate = index.data( AccountModel::CanRateRole ).toBool()
+                         || !author.isEmpty()
+                         || !desc.isEmpty(); // if it's Attica, or it has non-empty at least author or description
     if ( canRate )
     {
         textRect = QRect( leftTitleEdge, opt.rect.top() + PADDING, rightTitleEdge - leftTitleEdge, painter->fontMetrics().height() );
@@ -294,7 +300,6 @@ AccountDelegate::paint ( QPainter* painter, const QStyleOptionViewItem& option,
     painter->drawText( textRect, Qt::AlignVCenter | Qt::AlignLeft, title );
 
     // author
-    QString author = index.data( AccountModel::AuthorRole ).toString();
     int runningBottom = textRect.bottom();
     if ( !author.isEmpty() && canRate )
     {
@@ -310,7 +315,6 @@ AccountDelegate::paint ( QPainter* painter, const QStyleOptionViewItem& option,
     }
 
     // description
-    QString desc = index.data( AccountModel::DescriptionRole ).toString();
     const int descWidth = rightEdge - leftTitleEdge - PADDING;
     painter->setFont( descFont );
     const QRect descRect( leftTitleEdge, runningBottom + PADDING, descWidth, painter->fontMetrics().height() );
@@ -368,6 +372,23 @@ AccountDelegate::paint ( QPainter* painter, const QStyleOptionViewItem& option,
         painter->drawText( countRect, Qt::AlignLeft, count );
         //         runningEdge = authorRect.x();
     }
+    else //no rating, it's not attica, let's show other stuff...
+    {
+        QString versionString = index.data( AccountModel::VersionRole ).toString();
+
+        if ( !versionString.isEmpty() )
+        {
+            int runningEdge = textRect.left();
+            int pkgTop = runningBottom + PADDING;
+            int h = painter->fontMetrics().height();
+
+            QRect pkgRect( runningEdge, pkgTop, h, h );
+            painter->drawPixmap( pkgRect, TomahawkUtils::defaultPixmap( TomahawkUtils::ResolverBundle, TomahawkUtils::Original, pkgRect.size() ) );
+
+            QRect textRect( runningEdge + PADDING + h, pkgTop, painter->fontMetrics().width( versionString ), h );
+            painter->drawText( textRect, Qt::AlignLeft, versionString );
+        }
+    }
 
     // Title and description!
     return;
@@ -398,8 +419,10 @@ AccountDelegate::drawAccountList( QPainter* painter, QStyleOptionViewItemV4& opt
 
     for ( int i = 0; i < accts.size(); i++ )
     {
+        //FIXME: special case for twitter, remove for 0.8.0
+        if ( accts.at( i )->accountServiceName() != "Twitter" )
         // draw lightbulb and text
-        runningRightEdge = drawStatus( painter, QPointF( rightEdge - PADDING, current), accts.at( i ) );
+            runningRightEdge = drawStatus( painter, QPointF( rightEdge - PADDING, current), accts.at( i ) );
 
         const QString label = accts.at( i )->accountFriendlyName();
         const QPoint textTopLeft( runningRightEdge - PADDING - painter->fontMetrics().width( label ), current);
