@@ -2,6 +2,7 @@
  *
  *   Copyright 2010-2011, Christian Muehlhaeuser <muesli@tomahawk-player.org>
  *   Copyright 2010-2012, Jeff Mitchell <jeff@tomahawk-player.org>
+ *   Copyright 2013,      Teo Mrnjavac <teo@kde.org>
  *
  *   Tomahawk is free software: you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -19,6 +20,25 @@
 
 #include "Servent.h"
 
+#include "Result.h"
+#include "Source.h"
+#include "BufferIoDevice.h"
+#include "Connection.h"
+#include "ControlConnection.h"
+#include "database/Database.h"
+#include "database/DatabaseImpl.h"
+#include "StreamConnection.h"
+#include "SourceList.h"
+#include "sip/SipInfo.h"
+#include "sip/PeerInfo.h"
+#include "sip/SipPlugin.h"
+#include "PortFwdThread.h"
+#include "TomahawkSettings.h"
+#include "utils/TomahawkUtils.h"
+#include "utils/Logger.h"
+#include "accounts/AccountManager.h"
+
+
 #include <QtCore/QCoreApplication>
 #include <QtCore/QMutexLocker>
 #include <QtNetwork/QNetworkInterface>
@@ -30,21 +50,6 @@
 
 #include <boost/bind.hpp>
 
-#include "Result.h"
-#include "Source.h"
-#include "BufferIoDevice.h"
-#include "Connection.h"
-#include "ControlConnection.h"
-#include "database/Database.h"
-#include "database/DatabaseImpl.h"
-#include "StreamConnection.h"
-#include "SourceList.h"
-
-#include "PortFwdThread.h"
-#include "TomahawkSettings.h"
-#include "utils/TomahawkUtils.h"
-#include "utils/Logger.h"
-
 using namespace Tomahawk;
 
 Servent* Servent::s_instance = 0;
@@ -66,30 +71,36 @@ Servent::Servent( QObject* parent )
     s_instance = this;
 
     m_lanHack = qApp->arguments().contains( "--lanhack" );
+    m_noAuth = qApp->arguments().contains( "--noauth" );
+
     setProxy( QNetworkProxy::NoProxy );
 
     {
-    boost::function<QSharedPointer<QIODevice>(result_ptr)> fac =
-        boost::bind( &Servent::localFileIODeviceFactory, this, _1 );
-    this->registerIODeviceFactory( "file", fac );
+        // _1 = result, _2 = callback function for IODevice
+        IODeviceFactoryFunc fac = boost::bind( &Servent::localFileIODeviceFactory, this, _1, _2 );
+        this->registerIODeviceFactory( "file", fac );
     }
 
     {
-    boost::function<QSharedPointer<QIODevice>(result_ptr)> fac =
-        boost::bind( &Servent::remoteIODeviceFactory, this, _1 );
-    this->registerIODeviceFactory( "servent", fac );
+        IODeviceFactoryFunc fac = boost::bind( &Servent::remoteIODeviceFactory, this, _1, _2 );
+        this->registerIODeviceFactory( "servent", fac );
     }
 
     {
-    boost::function<QSharedPointer<QIODevice>(result_ptr)> fac =
-        boost::bind( &Servent::httpIODeviceFactory, this, _1 );
-    this->registerIODeviceFactory( "http", fac );
+        IODeviceFactoryFunc fac = boost::bind( &Servent::httpIODeviceFactory, this, _1, _2 );
+        this->registerIODeviceFactory( "http", fac );
+        this->registerIODeviceFactory( "https", fac );
     }
 }
 
 
 Servent::~Servent()
 {
+    tDebug() << Q_FUNC_INFO;
+
+    foreach ( ControlConnection* cc, m_controlconnections )
+        delete cc;
+
     if ( m_portfwd )
     {
         m_portfwd.data()->quit();
@@ -124,7 +135,7 @@ Servent::startListening( QHostAddress ha, bool upnp, int port )
     }
 
     TomahawkSettings::ExternalAddressMode mode = TomahawkSettings::instance()->externalAddressMode();
-    
+
     tLog() << "Servent listening on port" << m_port << "- servent thread:" << thread()
            << "- address mode:" << (int)( mode );
 
@@ -150,7 +161,7 @@ Servent::startListening( QHostAddress ha, bool upnp, int port )
             }
             // TODO check if we have a public/internet IP on this machine directly
             tLog() << "External address mode set to upnp...";
-            m_portfwd = QWeakPointer< PortFwdThread >( new PortFwdThread( m_port ) );
+            m_portfwd = QPointer< PortFwdThread >( new PortFwdThread( m_port ) );
             Q_ASSERT( m_portfwd );
             connect( m_portfwd.data(), SIGNAL( externalAddressDetected( QHostAddress, unsigned int ) ),
                                   SLOT( setExternalAddress( QHostAddress, unsigned int ) ) );
@@ -168,7 +179,7 @@ Servent::createConnectionKey( const QString& name, const QString &nodeid, const
     Q_ASSERT( this->thread() == QThread::currentThread() );
 
     QString _key = ( key.isEmpty() ? uuid() : key );
-    ControlConnection* cc = new ControlConnection( this, name );
+    ControlConnection* cc = new ControlConnection( this );
     cc->setName( name.isEmpty() ? QString( "KEY(%1)" ).arg( key ) : name );
     if ( !nodeid.isEmpty() )
         cc->setId( nodeid );
@@ -243,41 +254,175 @@ Servent::setExternalAddress( QHostAddress ha, unsigned int port )
 void
 Servent::registerOffer( const QString& key, Connection* conn )
 {
-    m_offers[key] = QWeakPointer<Connection>(conn);
+    m_offers[key] = QPointer<Connection>(conn);
 }
 
 
 void
 Servent::registerControlConnection( ControlConnection* conn )
 {
-    m_controlconnections.append( conn );
+    Q_ASSERT( conn );
+    tLog( LOGVERBOSE ) << Q_FUNC_INFO << conn->name();
+    m_controlconnections << conn;
+    m_connectedNodes << conn->id();
 }
 
 
 void
 Servent::unregisterControlConnection( ControlConnection* conn )
 {
-    QList<ControlConnection*> n;
-    foreach( ControlConnection* c, m_controlconnections )
-        if( c!=conn )
-            n.append( c );
+    Q_ASSERT( conn );
 
+    tLog( LOGVERBOSE ) << Q_FUNC_INFO << conn->name();
     m_connectedNodes.removeAll( conn->id() );
-    m_controlconnections = n;
+    m_controlconnections.removeAll( conn );
 }
 
 
 ControlConnection*
-Servent::lookupControlConnection( const QString& name )
+Servent::lookupControlConnection( const SipInfo& sipInfo )
 {
-    foreach( ControlConnection* c, m_controlconnections )
-        if( c->name() == name )
-            return c;
+    foreach ( ControlConnection* c, m_controlconnections )
+    {
+        tLog() << sipInfo.port() << c->peerPort() << sipInfo.host() << c->peerIpAddress().toString();
+        if ( sipInfo.port() == c->peerPort() && sipInfo.host() == c->peerIpAddress().toString() )
+             return c;
+    }
 
     return NULL;
 }
 
 
+void
+Servent::registerPeer( const Tomahawk::peerinfo_ptr& peerInfo )
+{
+    if ( peerInfo->hasControlConnection() )
+    {
+        peerInfoDebug( peerInfo ) << "already had control connection, doing nothing: " << peerInfo->controlConnection()->name();
+        tLog() << "existing control connection has following peers:";
+        foreach ( const peerinfo_ptr& otherPeerInfo, peerInfo->controlConnection()->peerInfos() )
+        {
+            peerInfoDebug( otherPeerInfo );
+        }
+
+        tLog() << "end peers";
+        return;
+    }
+
+    if ( peerInfo->type() == Tomahawk::PeerInfo::Local )
+    {
+        peerInfoDebug(peerInfo) << "we need to establish the connection now... thinking";
+        if ( !connectedToSession( peerInfo->sipInfo().nodeId() ) )
+        {
+            connectToPeer( peerInfo );
+        }
+        else
+        {
+//            FIXME: do we need to port this?!
+
+//            qDebug() << "Already connected to" << host; // so peerInfo was 0 before
+//            qDebug() << "They connected to us and we don't have a PeerInfo object, created one...";
+//            m_peersOnline.append( peerInfo );
+
+//            // attach to control connection
+//            ControlConnection* conn = Servent::instance()->lookupControlConnection( sipInfo );
+
+//            // we're connected to this nodeid, so we should find a control connection for this sipinfo, no?
+//            Q_ASSERT( conn );
+
+//            conn->addPeerInfo( peerInfo );
+        }
+    }
+    else
+    {
+        SipInfo info;
+        QString peerId = peerInfo->id();
+        QString key = uuid();
+        ControlConnection* conn = new ControlConnection( this );
+
+        const QString& nodeid = Database::instance()->impl()->dbid();
+        conn->setName( peerInfo->contactId() );
+        conn->setId( nodeid );
+        conn->addPeerInfo( peerInfo );
+
+        if ( visibleExternally() )
+        {
+            registerOffer( key, conn );
+            info.setVisible( true );
+            info.setHost( externalAddress() );
+            info.setPort( externalPort() );
+            info.setKey( key );
+            info.setNodeId( nodeid );
+
+            tDebug() << "Asking them (" << peerInfo->id() << ") to connect to us:" << info;
+        }
+        else
+        {
+            info.setVisible( false );
+            tDebug() << "We are not visible externally:" << info;
+        }
+
+        peerInfo->sendLocalSipInfo( info );
+
+        handleSipInfo( peerInfo );
+        connect( peerInfo.data(), SIGNAL( sipInfoChanged() ), SLOT( onSipInfoChanged() ) );
+    }
+}
+
+
+void
+Servent::onSipInfoChanged()
+{
+    Tomahawk::PeerInfo* peerInfo = qobject_cast< Tomahawk::PeerInfo* >( sender() );
+
+    if ( !peerInfo )
+        return;
+
+    handleSipInfo( peerInfo->weakRef().toStrongRef() );
+}
+
+
+void Servent::handleSipInfo( const Tomahawk::peerinfo_ptr& peerInfo )
+{
+    tLog() << Q_FUNC_INFO << peerInfo->id() << peerInfo->sipInfo();
+
+    SipInfo info = peerInfo->sipInfo();
+    if ( !info.isValid() )
+        return;
+
+    /*
+        If only one party is externally visible, connection is obvious
+        If both are, peer with lowest IP address initiates the connection.
+
+        This avoids dupe connections.
+    */
+    if ( info.isVisible() )
+    {
+        if ( !visibleExternally() ||
+             externalAddress() < info.host() ||
+             ( externalAddress() == info.host() && externalPort() < info.port() ) )
+        {
+
+            tDebug() << "Initiate connection to" << peerInfo->id() << "at" << info.host() << "peer of:" << peerInfo->sipPlugin()->account()->accountFriendlyName();
+            connectToPeer( peerInfo );
+        }
+        else
+        {
+            tDebug() << Q_FUNC_INFO << "They should be conecting to us...";
+        }
+    }
+    else
+    {
+        tDebug() << Q_FUNC_INFO << "They are not visible, doing nothing atm";
+
+        if ( !visibleExternally() )
+        {
+            if ( peerInfo->controlConnection() )
+                delete peerInfo->controlConnection();
+        }
+    }
+}
+
 void
 Servent::incomingConnection( int sd )
 {
@@ -289,7 +434,7 @@ Servent::incomingConnection( int sd )
     sock->moveToThread( thread() );
     sock->_disowned = false;
     sock->_outbound = false;
-    if( !sock->setSocketDescriptor( sd ) )
+    if ( !sock->setSocketDescriptor( sd ) )
     {
         Q_ASSERT( false );
         return;
@@ -304,35 +449,41 @@ void
 Servent::readyRead()
 {
     Q_ASSERT( this->thread() == QThread::currentThread() );
-    QWeakPointer< QTcpSocketExtra > sock = (QTcpSocketExtra*)sender();
+    QPointer< QTcpSocketExtra > sock = (QTcpSocketExtra*)sender();
 
-    if( sock.isNull() || sock.data()->_disowned )
+    if ( sock.isNull() || sock.data()->_disowned )
     {
         return;
     }
 
-    if( sock.data()->_msg.isNull() )
+    if ( sock.data()->_msg.isNull() )
     {
         char msgheader[ Msg::headerSize() ];
-        if( sock.data()->bytesAvailable() < Msg::headerSize() )
+        if ( sock.data()->bytesAvailable() < Msg::headerSize() )
             return;
 
         sock.data()->read( (char*) &msgheader, Msg::headerSize() );
         sock.data()->_msg = Msg::begin( (char*) &msgheader );
     }
 
-    if( sock.data()->bytesAvailable() < sock.data()->_msg->length() )
+    if ( sock.data()->bytesAvailable() < sock.data()->_msg->length() )
         return;
 
     QByteArray ba = sock.data()->read( sock.data()->_msg->length() );
     sock.data()->_msg->fill( ba );
-    Q_ASSERT( sock.data()->_msg->is( Msg::JSON ) );
+
+    if ( !sock.data()->_msg->is( Msg::JSON ) )
+    {
+        tDebug() << ba;
+        tDebug() << sock.data()->_msg->payload();
+        Q_ASSERT( sock.data()->_msg->is( Msg::JSON ) );
+    }
 
     ControlConnection* cc = 0;
     bool ok;
     QString key, conntype, nodeid, controlid;
     QVariantMap m = parser.parse( sock.data()->_msg->payload(), &ok ).toMap();
-    if( !ok )
+    if ( !ok )
     {
         tDebug() << "Invalid JSON on new connection, aborting";
         goto closeconnection;
@@ -344,32 +495,66 @@ Servent::readyRead()
     controlid = m.value( "controlid" ).toString();
 
     tDebug( LOGVERBOSE ) << "Incoming connection details:" << m;
-
-    if( !nodeid.isEmpty() ) // only control connections send nodeid
+    if ( !nodeid.isEmpty() ) // only control connections send nodeid
     {
         bool dupe = false;
         if ( m_connectedNodes.contains( nodeid ) )
-            dupe = true;
-
-        foreach( ControlConnection* con, m_controlconnections )
         {
-            tLog( LOGVERBOSE ) << "known connection:" << con->id() << con->source()->friendlyName();
-            if( con->id() == nodeid )
+            tDebug( LOGVERBOSE ) << Q_FUNC_INFO << "Connected nodes contains it.";
+            dupe = true;
+        }
+
+        foreach ( ControlConnection* con, m_controlconnections )
+        {
+            Q_ASSERT( con );
+
+            tLog( LOGVERBOSE ) << Q_FUNC_INFO << "Known connection:" << con->id();
+            if ( con->id() == nodeid )
             {
                 dupe = true;
                 break;
             }
         }
 
-        if ( dupe )
+        // for zeroconf there might be no offer, that case is handled later
+        ControlConnection* ccMatch = qobject_cast< ControlConnection* >( m_offers.value( key ).data() );
+        if ( dupe && ccMatch )
         {
             tLog() << "Duplicate control connection detected, dropping:" << nodeid << conntype;
+
+            tDebug() << "PEERINFO: to be dropped connection has following peers";
+            foreach ( const peerinfo_ptr& currentPeerInfo, ccMatch->peerInfos() )
+            {
+                peerInfoDebug( currentPeerInfo );
+            }
+
+            foreach ( ControlConnection* keepConnection, m_controlconnections )
+            {
+                Q_ASSERT( keepConnection );
+
+                if ( keepConnection->id() == nodeid )
+                {
+                    tDebug() << "Keep connection" << keepConnection->name() << "with following peers";
+                    foreach ( const peerinfo_ptr& currentPeerInfo, keepConnection->peerInfos() )
+                        peerInfoDebug( currentPeerInfo );
+
+                    tDebug() << "Add these peers now";
+                    foreach ( const peerinfo_ptr& currentPeerInfo, ccMatch->peerInfos() )
+                    {
+                        tDebug() << "Adding" << currentPeerInfo->id();
+                        keepConnection->addPeerInfo( currentPeerInfo );
+                    }
+                    tDebug() << "Done adding.";
+                }
+            }
             goto closeconnection;
         }
     }
 
-    foreach( ControlConnection* con, m_controlconnections )
+    foreach ( ControlConnection* con, m_controlconnections )
     {
+        Q_ASSERT( con );
+
         if ( con->id() == controlid )
         {
             cc = con;
@@ -381,7 +566,7 @@ Servent::readyRead()
     if ( conntype == "accept-offer" || conntype == "push-offer" )
     {
         sock.data()->_msg.clear();
-        tDebug( LOGVERBOSE ) << Q_FUNC_INFO << key << nodeid << "socket peer address = " << sock.data()->peerAddress() << "socket peer name = " << sock.data()->peerName();
+        tDebug( LOGVERBOSE ) << Q_FUNC_INFO << key << nodeid << "socket peer address =" << sock.data()->peerAddress() << "socket peer name =" << sock.data()->peerName();
         Connection* conn = claimOffer( cc, nodeid, key, sock.data()->peerAddress() );
         if ( !conn )
         {
@@ -398,11 +583,13 @@ Servent::readyRead()
             tLog() << "Socket has become invalid, possibly took too long to make an ACL decision, key:" << key << nodeid;
             goto closeconnection;
         }
-        tDebug( LOGVERBOSE ) << "claimOffer OK:" << key << nodeid;        
-        
-        m_connectedNodes << nodeid;
-        if( !nodeid.isEmpty() )
+        tDebug( LOGVERBOSE ) << "claimOffer OK:" << key << nodeid;
+
+        if ( !nodeid.isEmpty() )
+        {
             conn->setId( nodeid );
+            registerControlConnection( qobject_cast<ControlConnection*>(conn) );
+        }
 
         handoverSocket( conn, sock.data() );
         return;
@@ -427,7 +614,7 @@ Servent::createParallelConnection( Connection* orig_conn, Connection* new_conn,
 {
     tDebug( LOGVERBOSE ) << Q_FUNC_INFO << ", key:" << key << thread() << orig_conn;
     // if we can connect to them directly:
-    if( orig_conn && orig_conn->outbound() )
+    if ( orig_conn && orig_conn->outbound() )
     {
         connectToPeer( orig_conn->socket()->peerAddress().toString(),
                        orig_conn->peerPort(),
@@ -458,7 +645,7 @@ Servent::socketConnected()
 {
     QTcpSocketExtra* sock = (QTcpSocketExtra*)sender();
 
-    tDebug( LOGVERBOSE ) << Q_FUNC_INFO << thread() << "socket: " << sock << ", hostaddr: " << sock->peerAddress() << ", hostname: " << sock->peerName();
+    tDebug( LOGVERBOSE ) << Q_FUNC_INFO << thread() << "socket:" << sock << ", hostaddr:" << sock->peerAddress() << ", hostname:" << sock->peerName();
 
     if ( sock->_conn.isNull() )
     {
@@ -467,7 +654,7 @@ Servent::socketConnected()
         tDebug( LOGVERBOSE ) << Q_FUNC_INFO << "Socket's connection was null, could have timed out or been given an invalid address";
         return;
     }
-    
+
     Connection* conn = sock->_conn.data();
     handoverSocket( conn, sock );
 }
@@ -526,31 +713,89 @@ Servent::socketError( QAbstractSocket::SocketError e )
 
 
 void
-Servent::connectToPeer( const QString& ha, int port, const QString &key, const QString& name, const QString& id )
+Servent::connectToPeer( const peerinfo_ptr& peerInfo )
 {
     Q_ASSERT( this->thread() == QThread::currentThread() );
 
-    ControlConnection* conn = new ControlConnection( this, ha );
+    SipInfo sipInfo = peerInfo->sipInfo();
+
+    peerInfoDebug( peerInfo ) << "connectToPeer: search for already established connections to the same nodeid:" << m_controlconnections.count() << "connections";
+    if ( peerInfo->controlConnection() )
+        delete peerInfo->controlConnection();
+
+    bool isDupe = false;
+    ControlConnection* conn = 0;
+    // try to find a ControlConnection with the same SipInfo, then we dont need to try to connect again
+
+    foreach ( ControlConnection* c, m_controlconnections )
+    {
+        Q_ASSERT( c );
+
+        if ( c->id() == sipInfo.nodeId() )
+        {
+            conn = c;
+
+            foreach ( const peerinfo_ptr& currentPeerInfo, c->peerInfos() )
+            {
+                peerInfoDebug( currentPeerInfo ) << "Same object:" << ( peerInfo == currentPeerInfo ) << ( peerInfo.data() == currentPeerInfo.data() ) << ( peerInfo->debugName() == currentPeerInfo->debugName() );
+
+                if ( peerInfo == currentPeerInfo )
+                {
+                    isDupe = true;
+                    peerInfoDebug( currentPeerInfo ) << "Not adding, because it's a dupe: peerInfoCount remains the same" << conn->peerInfos().count();
+                    break;
+                }
+            }
+
+            if ( !c->peerInfos().contains( peerInfo ) )
+            {
+                c->addPeerInfo( peerInfo );
+//                peerInfoDebug(peerInfo) << "Adding " << peerInfo->debugName() << ", not a dupe... new peerInfoCount:" << c->peerInfos().count();
+//                foreach ( const peerinfo_ptr& kuh, c->peerInfos() )
+//                {
+//                    peerInfoDebug(peerInfo) << " ** " << kuh->debugName();
+//                }
+            }
+
+            break;
+        }
+    }
+
+    peerInfoDebug( peerInfo ) << "connectToPeer: found a match:" << ( conn ? conn->name() : "false" ) << "dupe:" << isDupe;
+
+    if ( isDupe )
+    {
+        peerInfoDebug( peerInfo ) << "it's a dupe, nothing to do here, returning and stopping processing: peerInfoCount:" << conn->peerInfos().count();
+    }
+
+    if ( conn )
+        return;
+
     QVariantMap m;
     m["conntype"]  = "accept-offer";
-    m["key"]       = key;
+    m["key"]       = sipInfo.key();
     m["port"]      = externalPort();
     m["nodeid"]    = Database::instance()->impl()->dbid();
 
+    peerInfoDebug(peerInfo) << "No match found, creating a new ControlConnection...";
+    conn = new ControlConnection( this );
+    conn->addPeerInfo( peerInfo );
     conn->setFirstMessage( m );
-    if( name.length() )
-        conn->setName( name );
-    if( id.length() )
-        conn->setId( id );
 
-    conn->setProperty( "nodeid", id );
+    if ( peerInfo->id().length() )
+        conn->setName( peerInfo->contactId() );
+    if ( sipInfo.nodeId().length() )
+        conn->setId( sipInfo.nodeId() );
 
-    connectToPeer( ha, port, key, conn );
+    conn->setProperty( "nodeid", sipInfo.nodeId() );
+
+    registerControlConnection( conn );
+    connectToPeer( sipInfo.host(), sipInfo.port(), sipInfo.key(), conn );
 }
 
 
 void
-Servent::connectToPeer( const QString& ha, int port, const QString &key, Connection* conn )
+Servent::connectToPeer( const QString& ha, int port, const QString& key, Connection* conn )
 {
     tDebug( LOGVERBOSE ) << "Servent::connectToPeer:" << ha << ":" << port
                          << thread() << QThread::currentThread();
@@ -565,7 +810,7 @@ Servent::connectToPeer( const QString& ha, int port, const QString &key, Connect
         return;
     }
 
-    if( key.length() && conn->firstMessage().isNull() )
+    if ( key.length() && conn->firstMessage().isNull() )
     {
         QVariantMap m;
         m["conntype"]  = "accept-offer";
@@ -620,24 +865,23 @@ Servent::reverseOfferRequest( ControlConnection* orig_conn, const QString& their
 Connection*
 Servent::claimOffer( ControlConnection* cc, const QString &nodeid, const QString &key, const QHostAddress peer )
 {
-    bool noauth = qApp->arguments().contains( "--noauth" );
-
     // magic key for stream connections:
-    if( key.startsWith( "FILE_REQUEST_KEY:" ) )
+    if ( key.startsWith( "FILE_REQUEST_KEY:" ) )
     {
         // check if the source IP matches an existing, authenticated connection
-        if ( !noauth && peer != QHostAddress::Any && !isIPWhitelisted( peer ) )
+        if ( !m_noAuth && peer != QHostAddress::Any && !isIPWhitelisted( peer ) )
         {
             bool authed = false;
-            foreach( ControlConnection* cc, m_controlconnections )
+            foreach ( ControlConnection* cc, m_controlconnections )
             {
-                if( cc->socket()->peerAddress() == peer )
+                tDebug() << Q_FUNC_INFO << "Probing:" << cc->name();
+                if ( cc->socket() && cc->socket()->peerAddress() == peer )
                 {
                     authed = true;
                     break;
                 }
             }
-            if( !authed )
+            if ( !authed )
             {
                 tLog() << "File transfer request rejected, invalid source IP";
                 return NULL;
@@ -649,13 +893,26 @@ Servent::claimOffer( ControlConnection* cc, const QString &nodeid, const QString
         return sc;
     }
 
-    if( key == "whitelist" ) // LAN IP address, check source IP
+    if ( key == "whitelist" ) // LAN IP address, check source IP
     {
-        if( isIPWhitelisted( peer ) )
+        if ( isIPWhitelisted( peer ) )
         {
             tDebug() << "Connection is from whitelisted IP range (LAN)";
-            Connection* conn = new ControlConnection( this, peer.toString() );
+            ControlConnection* conn = new ControlConnection( this );
             conn->setName( peer.toString() );
+
+            Tomahawk::Accounts::Account* account = Tomahawk::Accounts::AccountManager::instance()->zeroconfAccount();
+
+            // if we get this connection the account should exist and be enabled
+            Q_ASSERT( account );
+            Q_ASSERT( account->enabled() );
+
+            // this is terrible, actually there should be a way to let this be created by the zeroconf plugin
+            // because this way we rely on the ip being used as id in two totally different parts of the code
+            Tomahawk::peerinfo_ptr peerInfo = Tomahawk::PeerInfo::get( account->sipPlugin(), peer.toString(), Tomahawk::PeerInfo::AutoCreate );
+            peerInfo->setContactId( peer.toString() );
+            peerInfoDebug( peerInfo );
+            conn->addPeerInfo( peerInfo );
             return conn;
         }
         else
@@ -665,10 +922,10 @@ Servent::claimOffer( ControlConnection* cc, const QString &nodeid, const QString
         }
     }
 
-    if( m_offers.contains( key ) )
+    if ( m_offers.contains( key ) )
     {
-        QWeakPointer<Connection> conn = m_offers.value( key );
-        if( conn.isNull() )
+        QPointer<Connection> conn = m_offers.value( key );
+        if ( conn.isNull() )
         {
             // This can happen if it's a streamconnection, but the audioengine has
             // already closed the iodevice, causing the connection to be deleted before
@@ -678,14 +935,14 @@ Servent::claimOffer( ControlConnection* cc, const QString &nodeid, const QString
         }
 
         tDebug( LOGVERBOSE ) << Q_FUNC_INFO << "nodeid is: " << nodeid;
-        if( !nodeid.isEmpty() )
+        if ( !nodeid.isEmpty() )
         {
             // Used by the connection for the ACL check
             // If there isn't a nodeid it's not the first connection and will already have been stopped
             conn.data()->setProperty( "nodeid", nodeid );
         }
 
-        if( conn.data()->onceOnly() )
+        if ( conn.data()->onceOnly() )
         {
             m_offers.remove( key );
             return conn.data();
@@ -695,10 +952,10 @@ Servent::claimOffer( ControlConnection* cc, const QString &nodeid, const QString
             return conn.data()->clone();
         }
     }
-    else if ( noauth )
+    else if ( m_noAuth )
     {
         Connection* conn;
-        conn = new ControlConnection( this, peer );
+        conn = new ControlConnection( this );
         conn->setName( key );
         return conn;
     }
@@ -710,8 +967,9 @@ Servent::claimOffer( ControlConnection* cc, const QString &nodeid, const QString
 }
 
 
-QSharedPointer<QIODevice>
-Servent::remoteIODeviceFactory( const result_ptr& result )
+void
+Servent::remoteIODeviceFactory( const Tomahawk::result_ptr& result,
+                                boost::function< void ( QSharedPointer< QIODevice >& ) > callback )
 {
     QSharedPointer<QIODevice> sp;
 
@@ -720,12 +978,18 @@ Servent::remoteIODeviceFactory( const result_ptr& result )
     const QString fileId = parts.at( 1 );
     source_ptr s = SourceList::instance()->get( sourceName );
     if ( s.isNull() || !s->controlConnection() )
-        return sp;
+    {
+        callback( sp );
+        return;
+    }
 
     ControlConnection* cc = s->controlConnection();
     StreamConnection* sc = new StreamConnection( this, cc, fileId, result );
     createParallelConnection( cc, sc, QString( "FILE_REQUEST_KEY:%1" ).arg( fileId ) );
-    return sc->iodevice();
+
+    //boost::functions cannot accept temporaries as parameters
+    sp = sc->iodevice();
+    callback( sp );
 }
 
 
@@ -763,7 +1027,7 @@ Servent::printCurrentTransfers()
 {
     int k = 1;
 //    qDebug() << "~~~ Active file transfer connections:" << m_scsessions.length();
-    foreach( StreamConnection* i, m_scsessions )
+    foreach ( StreamConnection* i, m_scsessions )
     {
         qDebug() << k << ") " << i->id();
     }
@@ -777,13 +1041,13 @@ Servent::isIPWhitelisted( QHostAddress ip )
     tDebug( LOGVERBOSE ) << Q_FUNC_INFO << "Performing checks against ip" << ip.toString();
     typedef QPair< QHostAddress, int > range;
     QList< range > subnetEntries;
-    
+
     QList< QNetworkInterface > networkInterfaces = QNetworkInterface::allInterfaces();
-    foreach( QNetworkInterface interface, networkInterfaces )
+    foreach ( QNetworkInterface interface, networkInterfaces )
     {
         tDebug( LOGVERBOSE ) << Q_FUNC_INFO << "Checking interface" << interface.humanReadableName();
         QList< QNetworkAddressEntry > addressEntries = interface.addressEntries();
-        foreach( QNetworkAddressEntry addressEntry, addressEntries )
+        foreach ( QNetworkAddressEntry addressEntry, addressEntries )
         {
             tDebug( LOGVERBOSE ) << Q_FUNC_INFO << "Checking address entry with ip" << addressEntry.ip().toString() << "and prefix length" << addressEntry.prefixLength();
             if ( ip.isInSubnet( addressEntry.ip(), addressEntry.prefixLength() ) )
@@ -801,9 +1065,11 @@ Servent::isIPWhitelisted( QHostAddress ip )
 bool
 Servent::connectedToSession( const QString& session )
 {
-    foreach( ControlConnection* cc, m_controlconnections )
+    foreach ( ControlConnection* cc, m_controlconnections )
     {
-        if( cc->id() == session )
+        Q_ASSERT( cc );
+
+        if ( cc->id() == session )
             return true;
     }
 
@@ -815,8 +1081,8 @@ void
 Servent::triggerDBSync()
 {
     // tell peers we have new stuff they should sync
-    QList<source_ptr> sources = SourceList::instance()->sources();
-    foreach( const source_ptr& src, sources )
+    QList< source_ptr > sources = SourceList::instance()->sources();
+    foreach ( const source_ptr& src, sources )
     {
         // skip local source
         if ( src.isNull() || src->isLocal() )
@@ -825,49 +1091,66 @@ Servent::triggerDBSync()
         if ( src->controlConnection() && src->controlConnection()->dbSyncConnection() ) // source online?
             src->controlConnection()->dbSyncConnection()->trigger();
     }
+    emit dbSyncTriggered();
 }
 
 
 void
-Servent::registerIODeviceFactory( const QString &proto, boost::function<QSharedPointer<QIODevice>(Tomahawk::result_ptr)> fac )
+Servent::registerIODeviceFactory( const QString &proto,
+                                  IODeviceFactoryFunc fac )
 {
     m_iofactories.insert( proto, fac );
 }
 
 
-QSharedPointer<QIODevice>
-Servent::getIODeviceForUrl( const Tomahawk::result_ptr& result )
+void
+Servent::getIODeviceForUrl( const Tomahawk::result_ptr& result,
+                            boost::function< void ( QSharedPointer< QIODevice >& ) > callback )
 {
     QSharedPointer<QIODevice> sp;
 
     QRegExp rx( "^([a-zA-Z0-9]+)://(.+)$" );
     if ( rx.indexIn( result->url() ) == -1 )
-        return sp;
+    {
+        callback( sp );
+        return;
+    }
 
     const QString proto = rx.cap( 1 );
     if ( !m_iofactories.contains( proto ) )
-        return sp;
+    {
+        callback( sp );
+        return;
+    }
 
-    return m_iofactories.value( proto )( result );
+    //QtScriptResolverHelper::customIODeviceFactory is async!
+    m_iofactories.value( proto )( result, callback );
 }
 
 
-QSharedPointer<QIODevice>
-Servent::localFileIODeviceFactory( const Tomahawk::result_ptr& result )
+void
+Servent::localFileIODeviceFactory( const Tomahawk::result_ptr& result,
+                                   boost::function< void ( QSharedPointer< QIODevice >& ) > callback )
 {
     // ignore "file://" at front of url
     QFile* io = new QFile( result->url().mid( QString( "file://" ).length() ) );
     if ( io )
         io->open( QIODevice::ReadOnly );
 
-    return QSharedPointer<QIODevice>( io );
+    //boost::functions cannot accept temporaries as parameters
+    QSharedPointer< QIODevice > sp = QSharedPointer<QIODevice>( io );
+    callback( sp );
 }
 
 
-QSharedPointer<QIODevice>
-Servent::httpIODeviceFactory( const Tomahawk::result_ptr& result )
+void
+Servent::httpIODeviceFactory( const Tomahawk::result_ptr& result,
+                              boost::function< void ( QSharedPointer< QIODevice >& ) > callback )
 {
     QNetworkRequest req( result->url() );
     QNetworkReply* reply = TomahawkUtils::nam()->get( req );
-    return QSharedPointer<QIODevice>( reply, &QObject::deleteLater );
+
+    //boost::functions cannot accept temporaries as parameters
+    QSharedPointer< QIODevice > sp = QSharedPointer< QIODevice >( reply, &QObject::deleteLater );
+    callback( sp );
 }
