@@ -20,10 +20,10 @@
 #ifndef SEEKSLIDER_H
 #define SEEKSLIDER_H
 
-#include <QtGui/QSlider>
-
 #include "DllMacro.h"
 
+#include <QSlider>
+
 class QTimeLine;
 
 class DLLEXPORT SeekSlider : public QSlider
