@@ -59,6 +59,8 @@ WhatsHotWidget::WhatsHotWidget( QWidget* parent )
     , ui( new Ui::WhatsHotWidget )
     , m_sortedProxy( 0 )
     , m_workerThread( 0 )
+    , m_spinner( 0 )
+    , m_loading( true )
 {
     ui->setupUi( this );
 
@@ -75,7 +77,6 @@ WhatsHotWidget::WhatsHotWidget( QWidget* parent )
     m_sortedProxy->setFilterCaseSensitivity( Qt::CaseInsensitive );
 
     ui->breadCrumbLeft->setRootIcon( TomahawkUtils::defaultPixmap( TomahawkUtils::Charts, TomahawkUtils::Original ) );
-
     connect( ui->breadCrumbLeft, SIGNAL( activateIndex( QModelIndex ) ), SLOT( leftCrumbIndexChanged( QModelIndex ) ) );
 
     ui->tracksViewLeft->setHeaderHidden( true );
@@ -103,31 +104,35 @@ WhatsHotWidget::WhatsHotWidget( QWidget* parent )
 
     // Read last viewed charts, to be used as defaults
     m_currentVIds = TomahawkSettings::instance()->lastChartIds();
-    qDebug() << "Got last chartIds:" << m_currentVIds;
-
-    // TracksView is first shown, show spinner on that
-    // After fadeOut, charts are loaded
-    m_loadingSpinner =  new AnimatedSpinner( ui->tracksViewLeft );
-    m_loadingSpinner->fadeIn();
+    tDebug( LOGVERBOSE ) << "Re-loading last chartIds:" << m_currentVIds;
 
     MetaPlaylistInterface* mpl = new MetaPlaylistInterface();
     mpl->addChildInterface( ui->tracksViewLeft->playlistInterface() );
-    mpl->addChildInterface( ui->artistsViewLeft->playlistInterface() );
+    mpl->addChildInterface( ui->artistsViewLeft->proxyModel()->playlistInterface() );
     mpl->addChildInterface( ui->albumsView->playlistInterface() );
     m_playlistInterface = playlistinterface_ptr( mpl );
+
+    // Lets have a spinner until loaded
+    ui->breadCrumbLeft->setVisible( false );
+    ui->stackLeft->setCurrentIndex( 2 );
+    m_spinner = new AnimatedSpinner( ui->albumsView );
+    m_spinner->fadeIn();
+
 }
 
 
 WhatsHotWidget::~WhatsHotWidget()
 {
-    qDebug() << "Deleting whatshot";
+    tDebug( LOGVERBOSE ) << Q_FUNC_INFO;
+
     // Write the settings
-    qDebug() << "Writing chartIds to settings:" << m_currentVIds;
     TomahawkSettings::instance()->setLastChartIds( m_currentVIds );
+
     qDeleteAll( m_workers );
     m_workers.clear();
     m_workerThread->exit( 0 );
-    m_playlistInterface.clear();
+
+    delete m_spinner;
     delete ui;
 }
 
@@ -142,7 +147,7 @@ WhatsHotWidget::playlistInterface() const
 bool
 WhatsHotWidget::isBeingPlayed() const
 {
-    if ( AudioEngine::instance()->currentTrackPlaylist() == ui->artistsViewLeft->playlistInterface() )
+    if ( AudioEngine::instance()->currentTrackPlaylist() == ui->artistsViewLeft->proxyModel()->playlistInterface() )
         return true;
 
     if ( AudioEngine::instance()->currentTrackPlaylist() == ui->tracksViewLeft->playlistInterface() )
@@ -212,74 +217,13 @@ WhatsHotWidget::infoSystemInfo( Tomahawk::InfoSystem::InfoRequestData requestDat
     {
         case InfoSystem::InfoChartCapabilities:
         {
-            QStandardItem *rootItem= m_crumbModelLeft->invisibleRootItem();
-            QVariantMap defaults;
-            if ( returnedData.contains( "defaults" ) )
-                defaults = returnedData.take( "defaults" ).toMap();
-
-            // We need to take this from data
-            QString defaultSource = returnedData.take( "defaultSource" ).toString();
-            // Here, we dont want current sessions last view, but rather what was current on previus quit
-            QString lastSeen = TomahawkSettings::instance()->lastChartIds().value( "lastseen" ).toString();
-            if( !lastSeen.isEmpty() )
-                defaultSource = lastSeen;
-
-            // Merge defaults with current defaults, split the value in to a list
-            foreach( const QString&key, m_currentVIds.keys() )
-                defaults[ key ] = m_currentVIds.value( key ).toString().split( "/" );
-            qDebug() << "Defaults after merge" << defaults;
-            foreach ( const QString label, returnedData.keys() )
-            {
-                QStandardItem *childItem = parseNode( rootItem, label, returnedData[ label ] );
-                rootItem->appendRow( childItem );
-            }
-
-            // Set the default source
-            // Set the default chart for each source
-            if( !defaults.empty() )
-            {
-                for ( int i = 0; i < rootItem->rowCount(); i++ )
-                {
-                    QStandardItem* source = rootItem->child( i, 0 );
-                    if ( defaultSource.toLower() == source->text().toLower() )
-                    {
-                        source->setData( true, Breadcrumb::DefaultRole );
-                    }
-
-                    if ( defaults.contains( source->text().toLower() ) )
-                    {
-                        QStringList defaultIndices = defaults[ source->text().toLower() ].toStringList();
-                        QStandardItem* cur = source;
-
-                        foreach( const QString& index, defaultIndices )
-                        {
-                            // Go through the children of the current item, marking the default one as default
-                            for ( int k = 0; k < cur->rowCount(); k++ )
-                            {
-                                if ( cur->child( k, 0 )->text().toLower() == index.toLower() )
-                                {
-                                    cur = cur->child( k, 0 ); // this is the default, drill down into the default to pick the next default
-                                    cur->setData( true, Breadcrumb::DefaultRole );
-                                    break;
-                                }
-                            }
-                        }
-                    }
-                }
-            }
-
-            m_sortedProxy->setSourceModel( m_crumbModelLeft );
-            m_sortedProxy->sort( 0, Qt::AscendingOrder );
-            ui->breadCrumbLeft->setModel( m_sortedProxy );
-            break;
+            setViewData( returnedData );
         }
-
         case InfoSystem::InfoChart:
         {
-
-            if( returnedData.contains( "chart_error") )
+            if ( returnedData.contains( "chart_error" ) )
             {
-                tDebug( LOGVERBOSE ) << Q_FUNC_INFO << "Info came back with error!!";
+                tDebug( LOGVERBOSE ) << Q_FUNC_INFO << "Info came back with error!";
 
                 Tomahawk::InfoSystem::InfoStringHash criteria;
                 criteria.insert( "chart_refetch", returnedData[ "chart_source" ].value< QString >() );
@@ -301,7 +245,6 @@ WhatsHotWidget::infoSystemInfo( Tomahawk::InfoSystem::InfoRequestData requestDat
                 break;
 
             const QString type = returnedData[ "type" ].toString();
-
             if ( !returnedData.contains( type ) )
                 break;
 
@@ -318,7 +261,7 @@ WhatsHotWidget::infoSystemInfo( Tomahawk::InfoSystem::InfoRequestData requestDat
                 loader->setType( ChartDataLoader::Artist );
                 loader->setData( returnedData[ "artists" ].value< QStringList >() );
 
-                connect( loader, SIGNAL( artists( Tomahawk::ChartDataLoader*, QList< Tomahawk::artist_ptr > ) ), this, SLOT( chartArtistsLoaded( Tomahawk::ChartDataLoader*, QList< Tomahawk::artist_ptr > ) ) );
+                connect( loader, SIGNAL( artists( Tomahawk::ChartDataLoader*, QList< Tomahawk::artist_ptr > ) ), SLOT( chartArtistsLoaded( Tomahawk::ChartDataLoader*, QList< Tomahawk::artist_ptr > ) ) );
 
                 TreeModel* artistsModel = new TreeModel( ui->artistsViewLeft );
                 artistsModel->setMode( InfoSystemMode );
@@ -334,7 +277,7 @@ WhatsHotWidget::infoSystemInfo( Tomahawk::InfoSystem::InfoRequestData requestDat
                 loader->setType( ChartDataLoader::Album );
                 loader->setData( returnedData[ "albums" ].value< QList< Tomahawk::InfoSystem::InfoStringHash > >() );
 
-                connect( loader, SIGNAL( albums( Tomahawk::ChartDataLoader*, QList< Tomahawk::album_ptr > ) ), this, SLOT( chartAlbumsLoaded( Tomahawk::ChartDataLoader*, QList< Tomahawk::album_ptr > ) ) );
+                connect( loader, SIGNAL( albums( Tomahawk::ChartDataLoader*, QList< Tomahawk::album_ptr > ) ), SLOT( chartAlbumsLoaded( Tomahawk::ChartDataLoader*, QList< Tomahawk::album_ptr > ) ) );
 
                 PlayableModel* albumModel = new PlayableModel( ui->albumsView );
                 albumModel->startLoading();
@@ -349,7 +292,7 @@ WhatsHotWidget::infoSystemInfo( Tomahawk::InfoSystem::InfoRequestData requestDat
                 loader->setType( ChartDataLoader::Track );
                 loader->setData( returnedData[ "tracks" ].value< QList< Tomahawk::InfoSystem::InfoStringHash > >() );
 
-                connect( loader, SIGNAL( tracks( Tomahawk::ChartDataLoader*, QList< Tomahawk::query_ptr > ) ), this, SLOT( chartTracksLoaded( Tomahawk::ChartDataLoader*, QList< Tomahawk::query_ptr > ) ) );
+                connect( loader, SIGNAL( tracks( Tomahawk::ChartDataLoader*, QList< Tomahawk::query_ptr > ) ), SLOT( chartTracksLoaded( Tomahawk::ChartDataLoader*, QList< Tomahawk::query_ptr > ) ) );
 
                 PlaylistModel* trackModel = new PlaylistModel( ui->tracksViewLeft );
                 trackModel->startLoading();
@@ -370,12 +313,78 @@ WhatsHotWidget::infoSystemInfo( Tomahawk::InfoSystem::InfoRequestData requestDat
     }
 }
 
+void
+WhatsHotWidget::setViewData(const QVariantMap &data)
+{
+
+    QStandardItem* rootItem = m_crumbModelLeft->invisibleRootItem();
+    QVariantMap returnedData = data;
+
+    // We need to take this from data
+    QString defaultSource = returnedData.take( "defaultSource" ).toString();
+    QVariantMap defaults = returnedData.take( "defaults" ).toMap();
+    // Here, we dont want current sessions last view, but rather what was current on previus quit
+    QString lastSeen = TomahawkSettings::instance()->lastChartIds().value( "lastseen" ).toString();
+
+    if ( !lastSeen.isEmpty() )
+        defaultSource = lastSeen;
+
+    // Merge defaults with current defaults, split the value in to a list
+    foreach ( const QString& key, m_currentVIds.keys() )
+        defaults[ key ] = m_currentVIds.value( key ).toString().split( "/" );
+
+    foreach ( const QString& label, returnedData.keys() )
+    {
+        QStandardItem* childItem = parseNode( rootItem, label, returnedData[ label ] );
+        const QString id = label.toLower();
+
+        if ( id == defaultSource.toLower() )
+        {
+            qDebug() << "Setting source as default" << id;
+            childItem->setData( true, Breadcrumb::DefaultRole );
+        }
+        if ( defaults.contains( id ) )
+        {
+            QStringList defaultIndices = defaults[ id ].toStringList();
+            QStandardItem* cur = childItem;
+
+            foreach ( const QString& index, defaultIndices )
+            {
+                // Go through the children of the current item, marking the default one as default
+                for ( int k = 0; k < cur->rowCount(); k++ )
+                {
+                    if ( cur->child( k, 0 )->text().toLower() == index.toLower() )
+                    {
+                        qDebug() << "Setting child as default" << index;
+                        cur = cur->child( k, 0 ); // this is the default, drill down into the default to pick the next default
+                        cur->setData( true, Breadcrumb::DefaultRole );
+                        break;
+                    }
+                }
+            }
+        }
+        rootItem->appendRow( childItem );
+    }
+}
 
 void
 WhatsHotWidget::infoSystemFinished( QString target )
 {
-    Q_UNUSED( target );
-    m_loadingSpinner->fadeOut();
+    if( m_loading )
+    {
+        if ( target != s_whatsHotIdentifier )
+        {
+            return;
+        }
+
+        m_sortedProxy->setSourceModel( m_crumbModelLeft );
+        m_sortedProxy->sort( 0, Qt::AscendingOrder );
+        ui->breadCrumbLeft->setModel( m_sortedProxy );
+
+        m_spinner->fadeOut();
+        ui->breadCrumbLeft->setVisible( true );
+        m_loading = false;
+    }
 }
 
 
@@ -437,6 +446,7 @@ WhatsHotWidget::leftCrumbIndexChanged( QModelIndex index )
     criteria.insert( "chart_expires", QString::number( chartExpires ) );
     /// Remember to lower the source!
     criteria.insert( "chart_source",  index.data().toString().toLower() );
+
     Tomahawk::InfoSystem::InfoRequestData requestData;
     QVariantMap customData;
     customData.insert( "whatshot_side", "left" );
@@ -472,12 +482,12 @@ WhatsHotWidget::changeEvent( QEvent* e )
 
 
 QStandardItem*
-WhatsHotWidget::parseNode( QStandardItem* parentItem, const QString &label, const QVariant &data )
+WhatsHotWidget::parseNode( QStandardItem* parentItem, const QString& label, const QVariant& data )
 {
     Q_UNUSED( parentItem );
-//     tDebug( LOGVERBOSE ) << "WhatsHot: parsing " << label;
+//     tDebug( LOGVERBOSE ) << "WhatsHot: parsing" << label;
 
-    QStandardItem *sourceItem = new QStandardItem( label );
+    QStandardItem* sourceItem = new QStandardItem( label );
 
     if ( data.canConvert< QList< Tomahawk::InfoSystem::InfoStringHash > >() )
     {
@@ -485,7 +495,7 @@ WhatsHotWidget::parseNode( QStandardItem* parentItem, const QString &label, cons
 
         foreach ( Tomahawk::InfoSystem::InfoStringHash chart, charts )
         {
-            QStandardItem *childItem= new QStandardItem( chart[ "label" ] );
+            QStandardItem* childItem= new QStandardItem( chart[ "label" ] );
             childItem->setData( chart[ "id" ], Breadcrumb::ChartIdRole );
             childItem->setData( chart[ "expires" ], Breadcrumb::ChartExpireRole );
 
@@ -505,7 +515,7 @@ WhatsHotWidget::parseNode( QStandardItem* parentItem, const QString &label, cons
         QVariantMap dataMap = data.toMap();
         foreach ( const QString childLabel,dataMap.keys() )
         {
-            QStandardItem *childItem  = parseNode( sourceItem, childLabel, dataMap[ childLabel ] );
+            QStandardItem* childItem  = parseNode( sourceItem, childLabel, dataMap[ childLabel ] );
             sourceItem->appendRow( childItem );
         }
     }
@@ -515,13 +525,13 @@ WhatsHotWidget::parseNode( QStandardItem* parentItem, const QString &label, cons
 
         foreach ( const QVariant value, dataList )
         {
-            QStandardItem *childItem = new QStandardItem( value.toString() );
+            QStandardItem* childItem = new QStandardItem( value.toString() );
             sourceItem->appendRow( childItem );
         }
     }
     else
     {
-        QStandardItem *childItem = new QStandardItem( data.toString() );
+        QStandardItem* childItem = new QStandardItem( data.toString() );
         sourceItem->appendRow( childItem );
     }
     return sourceItem;
@@ -565,7 +575,7 @@ WhatsHotWidget::chartArtistsLoaded( ChartDataLoader* loader, const QList< artist
 
     if ( m_artistModels.contains( chartId ) )
     {
-        foreach( const artist_ptr& artist, artists )
+        foreach ( const artist_ptr& artist, artists )
         {
             m_artistModels[ chartId ]->addArtists( artist );
             m_artistModels[ chartId ]->finishLoading();
