@@ -25,6 +25,7 @@
 
 #include <boost/bind.hpp>
 
+#include "sourcetree/items/ScriptCollectionItem.h"
 #include "sourcetree/items/SourceTreeItem.h"
 #include "sourcetree/items/SourceItem.h"
 #include "sourcetree/items/GroupItem.h"
@@ -33,7 +34,7 @@
 #include "sourcetree/items/LovedTracksItem.h"
 #include "SourceList.h"
 #include "Playlist.h"
-#include "Collection.h"
+#include "collection/Collection.h"
 #include "Source.h"
 #include "ViewManager.h"
 #include "GlobalActionManager.h"
@@ -58,9 +59,22 @@ SourcesModel::SourcesModel( QObject* parent )
     appendGroups();
     onSourcesAdded( SourceList::instance()->sources() );
 
-    connect( SourceList::instance(), SIGNAL( sourceAdded( Tomahawk::source_ptr ) ), SLOT( onSourceAdded( Tomahawk::source_ptr ) ) );
-    connect( SourceList::instance(), SIGNAL( sourceRemoved( Tomahawk::source_ptr ) ), SLOT( onSourceRemoved( Tomahawk::source_ptr ) ) );
-    connect( ViewManager::instance(), SIGNAL( viewPageActivated( Tomahawk::ViewPage* ) ), this, SLOT( viewPageActivated( Tomahawk::ViewPage* ) ) );
+    connect( SourceList::instance(), SIGNAL( sourceAdded( Tomahawk::source_ptr ) ),
+             SLOT( onSourceAdded( Tomahawk::source_ptr ) ) );
+    connect( SourceList::instance(), SIGNAL( sourceRemoved( Tomahawk::source_ptr ) ),
+             SLOT( onSourceRemoved( Tomahawk::source_ptr ) ) );
+    connect( ViewManager::instance(), SIGNAL( viewPageActivated( Tomahawk::ViewPage* ) ),
+             this, SLOT( viewPageActivated( Tomahawk::ViewPage* ) ) );
+
+    foreach ( const collection_ptr& c, SourceList::instance()->scriptCollections() )
+    {
+        onScriptCollectionAdded( c );
+    }
+
+    connect( SourceList::instance(), SIGNAL( scriptCollectionAdded( Tomahawk::collection_ptr ) ),
+             this, SLOT( onScriptCollectionAdded( Tomahawk::collection_ptr ) ) );
+    connect( SourceList::instance(), SIGNAL( scriptCollectionRemoved( Tomahawk::collection_ptr ) ),
+             this, SLOT( onScriptCollectionRemoved( Tomahawk::collection_ptr ) ) );
 }
 
 
@@ -275,7 +289,7 @@ SourcesModel::flags( const QModelIndex& index ) const
 void
 SourcesModel::appendGroups()
 {
-    beginInsertRows( QModelIndex(), rowCount(), rowCount() + 3 );
+    beginInsertRows( QModelIndex(), rowCount(), rowCount() + 4 );
 
     GroupItem* browse = new GroupItem( this, m_rootItem, tr( "Browse" ), 0 );
     new HistoryItem( this, m_rootItem, tr( "Search History" ), 1 );
@@ -314,6 +328,8 @@ SourcesModel::appendGroups()
 
     m_collectionsGroup = new GroupItem( this, m_rootItem, tr( "Friends" ), 4 );
 
+    m_cloudGroup = new GroupItem( this, m_rootItem, tr( "Cloud" ), 5 );
+
     endInsertRows();
 }
 
@@ -492,6 +508,56 @@ SourcesModel::onSourceRemoved( const source_ptr& source )
 }
 
 
+void
+SourcesModel::onScriptCollectionAdded( const collection_ptr& collection )
+{
+    if ( m_scriptCollections.contains( collection ) )
+        return;
+
+    QModelIndex parent = indexFromItem( m_cloudGroup );
+    beginInsertRows( parent, rowCount( parent ), rowCount( parent ) );
+    ScriptCollectionItem* item = new ScriptCollectionItem( this,
+                                                 m_cloudGroup,
+                                                 collection );
+    endInsertRows();
+
+    m_scriptCollections.insert( collection, item );
+    m_cloudGroup->checkExpandedState();
+}
+
+
+void
+SourcesModel::onScriptCollectionRemoved( const collection_ptr& collection )
+{
+    SourceTreeItem* item = m_scriptCollections.value( collection );
+    int row = indexFromItem( item ).row();
+
+    QModelIndex parent = indexFromItem( m_cloudGroup );
+    beginRemoveRows( parent, row, row );
+    m_cloudGroup->removeChild( item );
+    endRemoveRows();
+
+    m_scriptCollectionPages.remove( collection );
+    m_scriptCollections.remove( collection );
+    item->deleteLater();
+}
+
+
+ViewPage*
+SourcesModel::scriptCollectionClicked( const Tomahawk::collection_ptr& collection )
+{
+    m_scriptCollectionPages.insert( collection, ViewManager::instance()->show( collection ) );
+    return m_scriptCollectionPages[ collection ];
+}
+
+
+ViewPage*
+SourcesModel::getScriptCollectionPage( const Tomahawk::collection_ptr& collection ) const
+{
+    return m_scriptCollectionPages[ collection ];
+}
+
+
 void
 SourcesModel::itemUpdated()
 {
